import { Subject } from 'rxjs';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';
import { __awaiter } from 'tslib';
import { InjectionToken, Inject, Optional, Directive, ElementRef, Input, NgZone, TemplateRef, ViewContainerRef, ContentChildren, Attribute, EventEmitter, HostListener, Output, Renderer2, LOCALE_ID, ChangeDetectorRef, Self, HostBinding, Pipe, Injectable, NgModule, defineInjectable } from '@angular/core';
import { FormGroupDirective, NgForm, NG_VALUE_ACCESSOR, NG_VALIDATORS, NgControl, NG_ASYNC_VALIDATORS } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const LANGUAGE_CODE = new InjectionToken('LanguageCode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class BaseTranslationPipe {
    /**
     * @param {?} language
     */
    constructor(language) {
        this.language = language;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        if (!this.translations[this.language]) {
            console.warn(`Translation file for '${this.language}' does not exist.`);
            return value;
        }
        return this.translations[this.language][value] || value;
    }
}
/** @nocollapse */
BaseTranslationPipe.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drag API.
 */
class DraggableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.dataFn = () => this.getData();
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    /**
     * @private
     * @return {?}
     */
    getData() {
        return this.anjDraggable;
    }
}
DraggableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDraggable]'
            },] }
];
/** @nocollapse */
DraggableDirective.ctorParameters = () => [
    { type: ElementRef }
];
DraggableDirective.propDecorators = {
    anjDraggable: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drop API.
 */
class DroppableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
}
DroppableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDroppable]'
            },] }
];
/** @nocollapse */
DroppableDirective.ctorParameters = () => [
    { type: ElementRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DomOutlet {
    /**
     * @param {?} targetElement
     */
    constructor(targetElement) {
        this.targetElement = targetElement;
    }
    /**
     * @template C
     * @param {?} template
     * @return {?}
     */
    attachTemplate(template) {
        if (this.template.zone.isStable) {
            this._viewRef = this.template.zone.run(() => {
                return template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            });
        }
        else {
            this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            this._viewRef.detectChanges();
        }
        this._viewRef.rootNodes.forEach(rootNode => this.targetElement.appendChild(rootNode));
        this._viewRef.markForCheck();
        this.setDisposeFn((() => {
            /** @type {?} */
            const index = template.viewContainerRef.indexOf(this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    }
    /**
     * @param {?} context
     * @return {?}
     */
    updateContext(context) {
        for (const prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
        this._viewRef.detectChanges();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    setDisposeFn(fn) {
        this.disposeFn = fn;
    }
    /**
     * @param {?} template
     * @return {?}
     */
    attach(template) {
        this.template = template;
        return this.attachTemplate(template);
    }
    /**
     * @return {?}
     */
    detach() {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    }
    /**
     * @param {?=} removeParent
     * @return {?}
     */
    dispose(removeParent = true) {
        this.disposeFn();
        if (removeParent && this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    }
    /**
     * @private
     * @return {?}
     */
    invokeDisposeFn() {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Template {
    /**
     * @param {?} templateDirective
     * @param {?=} context
     */
    constructor(templateDirective, context) {
        this.templateDirective = templateDirective;
        this.context = context;
    }
    /**
     * @return {?}
     */
    get origin() {
        return this.templateDirective.template.elementRef;
    }
    /**
     * @return {?}
     */
    get templateRef() {
        return this.templateDirective.template;
    }
    /**
     * @return {?}
     */
    get viewContainerRef() {
        return this.templateDirective.viewContainerRef;
    }
    /**
     * @return {?}
     */
    get zone() {
        return this.templateDirective.zone;
    }
    /**
     * @param {?} host
     * @param {?=} context
     * @return {?}
     */
    attach(host, context = this.context) {
        this.context = context;
        this.host = host;
        return host.attach(this);
    }
    /**
     * @return {?}
     */
    detach() {
        this.context = undefined;
        if (this.host) {
            /** @type {?} */
            const host = this.host;
            this.host = null;
            host.detach();
        }
    }
    /**
     * @param {?} host
     * @return {?}
     */
    setAttachedHost(host) {
        this.host = host;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TemplateDirective {
    /**
     * @param {?} _template
     * @param {?} _viewRef
     * @param {?} _zone
     */
    constructor(_template, _viewRef, _zone) {
        this._template = _template;
        this._viewRef = _viewRef;
        this._zone = _zone;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set anjTemplateOf(name) {
        this._name = name;
    }
    /**
     * @return {?}
     */
    get name() {
        return this._name;
    }
    /**
     * @return {?}
     */
    get template() {
        return this._template;
    }
    /**
     * @return {?}
     */
    get viewContainerRef() {
        return this._viewRef;
    }
    /**
     * @return {?}
     */
    get zone() {
        return this._zone;
    }
}
TemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTemplate]'
            },] }
];
/** @nocollapse */
TemplateDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: NgZone }
];
TemplateDirective.propDecorators = {
    anjTemplateOf: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class AnjTable {
    /**
     * @param {?} elRef
     */
    constructor(elRef) {
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    /**
     * @private
     * @param {?} row
     * @param {?} index
     * @param {?=} column
     * @return {?}
     */
    static rowContext(row, index, column) {
        return {
            $implicit: row,
            column,
            index
        };
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} context
     * @return {?}
     */
    static updateContext(outlet, context) {
        outlet.updateContext(context);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.outletRows.map((rowMap) => {
            for (const templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    }
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    /**
     * @param {?} cell
     * @param {?} offset
     * @param {?} col
     * @param {?} row
     * @param {?} rowIndex
     * @param {?} template
     * @return {?}
     */
    cellTemplate(cell, offset, col, row, rowIndex, template) {
        /** @type {?} */
        const rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        const outlet = rowMap[col.name];
        if (!cell) {
            rowMap[col.name] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (cell !== outlet.targetElement) {
                this.updateOutlet(outlet, template, cell, AnjTable.rowContext(row, rowIndex, col));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex, col));
            }
        }
        else {
            rowMap[col.name] = this.createOutlet(cell, template, AnjTable.rowContext(row, rowIndex, col));
            this.outletRows[rowIndex] = rowMap;
        }
        col.initCell(cell, offset);
    }
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    rowTemplate(templateName, targetEl, row, rowIndex) {
        // Dispose row template no longer used for that index
        this.outletRows[rowIndex] = this.filterRowTemplates(this.outletRows[rowIndex], templateName);
        /** @type {?} */
        const outlet = this.outletRows[rowIndex][templateName];
        if (!targetEl) {
            this.outletRows[rowIndex][templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            this.outletRows[rowIndex][templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = this.outletRows[rowIndex];
        }
    }
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    fromTemplate(templateName, targetEl, row, rowIndex) {
        /** @type {?} */
        const rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        const outlet = rowMap[templateName];
        if (!targetEl) {
            rowMap[templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    }
    /**
     * @private
     * @param {?} target
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createOutlet(target, templateName, context) {
        /** @type {?} */
        const outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    }
    /**
     * @private
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createTemplate(templateName, context) {
        /** @type {?} */
        const templateRef = this.templates.find(t => t.name === templateName);
        return new Template(templateRef, context);
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} context
     * @return {?}
     */
    updateOutlet(outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    }
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     * @private
     * @param {?=} rowMap
     * @param {?=} templateName
     * @return {?}
     */
    filterRowTemplates(rowMap = {}, templateName) {
        /** @type {?} */
        const updated = {};
        Object.keys(rowMap).forEach(k => {
            if (k === templateName) {
                updated[k] = rowMap[k];
            }
            else {
                rowMap[k].dispose(false);
            }
        });
        return updated;
    }
}
AnjTable.decorators = [
    { type: Directive, args: [{
                selector: 'anj-table:not([anjIgnore])',
            },] }
];
/** @nocollapse */
AnjTable.ctorParameters = () => [
    { type: ElementRef }
];
AnjTable.propDecorators = {
    templates: [{ type: ContentChildren, args: [TemplateDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const TransitionState = {
    Enter: 0,
    Leave: 1,
    EnterTo: 2,
    Inactive: 3,
};
TransitionState[TransitionState.Enter] = 'Enter';
TransitionState[TransitionState.Leave] = 'Leave';
TransitionState[TransitionState.EnterTo] = 'EnterTo';
TransitionState[TransitionState.Inactive] = 'Inactive';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@experimental
 */
class TransitionDirective {
    /**
     * @param {?} renderer
     * @param {?} element
     * @param {?} mode
     * @param {?} name
     */
    constructor(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleTransition(event) {
        if (event.target === this.element.nativeElement && TransitionDirective.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    initEnter() {
        this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
    }
    /**
     * @private
     * @return {?}
     */
    transitionIn() {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter`);
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.endTransition();
        }
    }
    /**
     * @private
     * @return {?}
     */
    transitionOut() {
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave`);
        }
        else {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(() => {
                    this.transitionIn();
                });
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    endTransition() {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
        }
    }
}
TransitionDirective.supportedTransitions = ['opacity', 'transform'];
TransitionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTransition]'
            },] }
];
/** @nocollapse */
TransitionDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
];
TransitionDirective.propDecorators = {
    anjTransition: [{ type: Input }],
    end: [{ type: Output }],
    handleTransition: [{ type: HostListener, args: ['transitiend', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValueAccessor {
    /**
     * @param {?} element
     * @param {?} parentForm
     * @param {?} parentGroup
     */
    constructor(element, parentForm, parentGroup) {
        this.element = element;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.parent) {
            this.formSub = this.parent.ngSubmit.subscribe(this.checkValidity.bind(this));
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.formSub) {
            this.formSub.unsubscribe();
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onChangeEvent(value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    }
    /**
     * @return {?}
     */
    onAnjBlur() {
        this.onTouched();
        this.checkValidity();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    }
    /**
     * @return {?}
     */
    checkValidity() {
        // TODO: this won't work for custom validators
        this.element.nativeElement.checkValidity();
    }
}
ValueAccessor.propDecorators = {
    valueChange: [{ type: Output }],
    onAnjBlur: [{ type: HostListener, args: ['anjBlur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class BooleanValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.element.nativeElement.checked = this.lastValue = value == null ? false : value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
        this.checkValidity();
    }
}
BooleanValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox, anj-toggle',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: BooleanValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
BooleanValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
BooleanValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target.checked'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@dynamic
 * @abstract
 */
class AnjunaValidators {
    /**
     * @param {?} c
     * @return {?}
     */
    static date(c) {
        if (!c.value) {
            return;
        }
        /** @type {?} */
        let isValid = true;
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static requiredCheck(c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static unique(c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        /** @type {?} */
        const testArray = c.value.filter(Boolean);
        /** @type {?} */
        const isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    }
}
AnjunaValidators.minDate = (min) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value < new Date(min)) {
            return {
                min: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};
AnjunaValidators.maxDate = (max) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value > new Date(max)) {
            return {
                max: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class DateValidatorDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.date(c);
    }
}
DateValidatorDirective.decorators = [
    { type: Directive, args: [{
                selector: '[date][ngModel],[date][formControl]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: DateValidatorDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TIME_TIMEZONE_STRING = new InjectionToken('TimeTimezoneString');
/** @type {?} */
const TIME_MERIDIEM = new InjectionToken('TimeMeridiem');
/** @type {?} */
const DATE_TIMEZONE_OVERRIDE = new InjectionToken('DateTimezoneOverride');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const STANDARD_LOCALES = {
    'en-US': 'M/d/yyyy',
    'sw-KE': 'M/d/yyyy',
    'en-ZW': 'M/d/yyyy',
    'en-PH': 'M/d/yyyy',
    'rw-RW': 'M/d/yyyy',
    'moh-CA': 'M/d/yyyy',
    'fil-PH': 'M/d/yyyy',
    'ne-NP': 'M/d/yyyy',
    'es-US': 'M/d/yyyy',
    'sq-AL': 'yyyy/M/d',
    'bg-BG': 'yyyy/M/d',
    'sr-BA': 'yyyy/M/d',
    'fr-CA': 'yyyy/M/d',
    'zh-CN': 'yyyy/M/d',
    'zh-HK': 'yyyy/M/d',
    'hu-HU': 'yyyy/M/d',
    'ja-JP': 'yyyy/M/d',
    'ko-KR': 'yyyy/M/d',
    'lt-LT': 'yyyy/M/d',
    'sv-SE': 'yyyy/M/d',
    'zh-TW': 'yyyy/M/d',
    'en-ZA': 'yyyy/M/d',
    'en-CA': 'yyyy/M/d',
    'ug-CN': 'yyyy/M/d'
};
/**
 * @abstract
 * @template T
 */
class DateAdapter {
}
// Default adapter, will accept String or Native Date
class NativeDateAdapter {
    /**
     * @param {?=} _locale
     * @param {?=} _timezone
     */
    constructor(_locale, _timezone) {
        this._locale = _locale;
        this._timezone = _timezone;
        // TODO: Depending on i10n strategy, languageCode/locale may make more sense as props on the WC than injected tokens..
        if (!this._locale) {
            this._locale = 'en-US';
        }
        this.timezone = _timezone;
        if (!this.timezone) {
            console.warn('Timezone is not set, is defaulting to system timezone. DATE_TIMEZONE_OVERRIDE may be set at the Module level.');
        }
    }
    // Timezone-override aware date conversion
    /**
     * @param {?} input
     * @return {?}
     */
    convertToDate(input) {
        if (!input || !this.newDateFromValue(input)) {
            return null;
        }
        /** @type {?} */
        const localDateString = new Date(input).toLocaleDateString();
        /** @type {?} */
        const tzTime = new Date(localDateString).toLocaleString(undefined, { timeZone: this.timezone });
        /** @type {?} */
        const localDate = Number(new Date(input));
        /** @type {?} */
        const tzDate = Number(new Date(tzTime));
        /** @type {?} */
        let convertedDate;
        if (localDate >= tzDate) {
            /** @type {?} */
            const variance = localDate - tzDate;
            convertedDate = new Date(localDate + variance);
        }
        else {
            /** @type {?} */
            const variance = tzDate - localDate;
            convertedDate = new Date(localDate - variance);
        }
        return convertedDate;
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    dayOfFirst(year, month) {
        return new Date(year, month, 1).getDay();
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    daysInMonth(year, month) {
        return new Date(year, month + 1, 0).getDate();
    }
    /**
     * @param {?} evalDate
     * @param {?} dateRangeParams
     * @return {?}
     */
    isDisabled(evalDate, dateRangeParams) {
        if (!dateRangeParams) {
            return false;
        }
        /** @type {?} */
        const dayIsBeforeMinDate = (dateRangeParams.minDate) ? evalDate < dateRangeParams.minDate : false;
        /** @type {?} */
        const dayIsAfterMaxDate = (dateRangeParams.maxDate) ? evalDate > dateRangeParams.maxDate : false;
        return dayIsBeforeMinDate || dayIsAfterMaxDate;
    }
    /**
     * @param {?} year
     * @param {?} month
     * @return {?}
     */
    getMonthInfo(year, month) {
        return new Date(year, month, 15);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    month(date) {
        return date.getMonth();
    }
    /**
     * @param {?=} year
     * @param {?=} month
     * @param {?=} day
     * @return {?}
     */
    newDateFromArray(year, month, day) {
        return new Date(year, month, day);
    }
    /**
     * @param {?=} value
     * @return {?}
     */
    newDateFromValue(value) {
        /** @type {?} */
        const newValue = (value) ? new Date(value) : new Date();
        return (newValue.toString() === 'Invalid Date') ? null : newValue;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderDate(date) {
        return localize(this._locale, date, { weekday: 'short', month: 'short', day: 'numeric' });
    }
    /**
     * @param {?} date
     * @return {?}
     */
    prepHeaderYear(date) {
        return localize(this._locale, date, { year: 'numeric' });
    }
    /**
     * @param {?} _calendarDay
     * @param {?} _comparator
     * @return {?}
     */
    sameDay(_calendarDay, _comparator) {
        return _calendarDay.day === _comparator.getDate()
            && _calendarDay.month === _comparator.getMonth()
            && _calendarDay.year === _comparator.getFullYear();
    }
    /**
     * @param {?} date
     * @return {?}
     */
    year(date) {
        return date.getFullYear();
    }
    /**
     * @return {?}
     */
    weekdays() {
        return getWeekdays(this._locale);
    }
    /**
     * @return {?}
     */
    months() {
        return getMonths(this._locale);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    localize(date) {
        return new Date(date).toLocaleDateString(this._locale, { month: '2-digit', day: '2-digit', year: 'numeric' });
    }
    /**
     * @param {?} input
     * @return {?}
     */
    prepDateStringOutput(input) {
        if (!input) {
            return null;
        }
        else if (typeof input === 'string') {
            return input;
        }
        else if (this.timezone) {
            // Timezone-override aware formatting
            /** @type {?} */
            const tzTime = new Date(input).toLocaleString(undefined, { timeZone: this.timezone });
            return new Date(tzTime).toLocaleDateString(this._locale);
        }
        else {
            return this.localize(input);
        }
    }
    /**
     * @param {?} input
     * @return {?}
     */
    processDateInput(input) {
        if (!input) {
            return null;
        }
        if (typeof input === 'string') {
            // Remove any time notation by splitting on spaces or commas
            /** @type {?} */
            const regexSplit = input.split(/[ ,]+/g).filter(Boolean);
            /** @type {?} */
            const dateSegments = parseInput(regexSplit[0], this._locale);
            if (Array.isArray(dateSegments.splitDate) && dateSegments.splitDate.length === 3 && dateSegments.year) {
                return new Date(parseInt(dateSegments.year, 10), parseInt(dateSegments.month, 10) - 1, parseInt(dateSegments.day, 10));
            }
            else {
                return new Date(input);
            }
        }
        else {
            return input;
        }
    }
}
/** @nocollapse */
NativeDateAdapter.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] }
];
/**
 * @param {?} locale
 * @return {?}
 */
function getWeekdays(locale) {
    /** @type {?} */
    const weekdays = [];
    for (let i = 0; i < 7; i++) {
        /** @type {?} */
        const date = new Date();
        /** @type {?} */
        const testDate = date.setDate(date.getDate() - date.getDay() + i);
        weekdays.push(localize(locale, new Date(testDate), { weekday: 'short' }));
    }
    return weekdays;
}
/**
 * @param {?} locale
 * @return {?}
 */
function getMonths(locale) {
    /** @type {?} */
    const months = [];
    for (let i = 0; i < 12; i++) {
        /** @type {?} */
        const testDate = new Date().setMonth(i);
        months.push(localize(locale, new Date(testDate), { month: 'short' }));
    }
    return months;
}
/**
 * @param {?=} locale
 * @param {?=} date
 * @param {?=} options
 * @return {?}
 */
function localize(locale = 'en-US', date, options) {
    return new Intl.DateTimeFormat(locale, options).format(date);
}
/* Currently only support shortDate strings (with non-alphanumeric separators) of:
    {
        Month-Day-Year: if included in STANDARD_LOCALES,
        Year-Month-Day: if included in STANDARD_LOCALES,
        Day-Month-Year: for all locales outside of STANDARD_LOCALES
    }
 */
/**
 * @param {?} input
 * @param {?} locale
 * @return {?}
 */
function parseInput(input, locale) {
    /** @type {?} */
    let format = STANDARD_LOCALES[locale];
    // Covers case of partial locale code provided
    if (!format) {
        /** @type {?} */
        const localeKey = Object.keys(STANDARD_LOCALES).find(key => key.includes(locale));
        format = STANDARD_LOCALES[localeKey];
    }
    // Split string on any non-alphanumeric characters, not including spaces
    /** @type {?} */
    const regexSplit = input.split(/[^\w\s]+/g).filter(Boolean);
    // Parse digit-only string, or use the regexSplit value
    /** @type {?} */
    const splitDate = (regexSplit.length <= 1 && input.length === 8) ? parseDigitOnlyDate(input, format) : regexSplit;
    /** @type {?} */
    const dateSegments = {
        splitDate,
        year: '',
        month: '',
        day: ''
    };
    /** @type {?} */
    const splitDateCp = splitDate.slice(0);
    // Set year
    dateSegments.year = setYearSegment(splitDateCp);
    // Set Day and Month
    Object.assign(dateSegments, setMonthAndDaySegments(splitDateCp, format));
    return dateSegments;
}
/**
 * @param {?} splitDateArray
 * @return {?}
 */
function setYearSegment(splitDateArray) {
    // Find year
    /** @type {?} */
    const year = splitDateArray.find(segment => segment.length === 4);
    // Remove year from working array
    /** @type {?} */
    const yearIndex = splitDateArray.indexOf(year);
    splitDateArray.splice(yearIndex, 1);
    return year;
}
/**
 * @param {?} splitDateArray
 * @param {?} format
 * @return {?}
 */
function setMonthAndDaySegments(splitDateArray, format) {
    /** @type {?} */
    const segments = {
        day: '',
        month: ''
    };
    // See if we can determine which segment is the days segment
    /** @type {?} */
    const assumedDay = splitDateArray.find(segment => parseInt(segment, 10) > 12);
    if (assumedDay) {
        segments.day = assumedDay;
        /** @type {?} */
        const dayIndex = splitDateArray.indexOf(assumedDay);
        splitDateArray.splice(dayIndex, 1);
        segments.month = splitDateArray[0];
        // Otherwise, make a reasonable guess
    }
    else {
        switch (format) {
            case 'M/d/yyyy':
            case 'yyyy/M/d':
                segments.month = splitDateArray[0];
                segments.day = splitDateArray[1];
                break;
            // Assume 'dd/MM/yyyy' for anything else
            default:
                segments.month = splitDateArray[1];
                segments.day = splitDateArray[0];
                break;
        }
    }
    return segments;
}
/**
 * @param {?} input
 * @param {?} format
 * @return {?}
 */
function parseDigitOnlyDate(input, format) {
    /** @type {?} */
    let splitDate;
    switch (format) {
        case 'yyyy/M/d':
            splitDate = [
                input.slice(0, 4),
                input.slice(4, 6),
                input.slice(6)
            ];
            break;
        // Assume 'dd/MM/yyyy' or 'M/d/yyyy' for anything else
        default:
            splitDate = [
                input.slice(0, 2),
                input.slice(2, 4),
                input.slice(4)
            ];
            break;
    }
    return splitDate;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatePickerDirective {
    /**
     * @param {?} renderer
     * @param {?} _element
     * @param {?} changeRef
     * @param {?} dateAdapter
     * @param {?=} ngControl
     * @param {?=} _locale
     */
    constructor(renderer, _element, changeRef, dateAdapter, ngControl, _locale) {
        this.renderer = renderer;
        this._element = _element;
        this.changeRef = changeRef;
        this.dateAdapter = dateAdapter;
        this.ngControl = ngControl;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
        this.valueChange = new EventEmitter();
        this.adapterStrategy = dateAdapter;
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    // Custom input to return inner value, but display masked value
    /**
     * @return {?}
     */
    get value() { return this.innerDateValue; }
    /**
     * @param {?} dateValue
     * @return {?}
     */
    set value(dateValue) {
        if (dateValue) {
            this.outerValue = this.adapterStrategy.prepDateStringOutput(dateValue);
            this.changeRef.markForCheck();
        }
    }
    // AdapterStrategy will be set by the DateAdapter injected provider. This is for an optional override.
    /**
     * @param {?} adapter
     * @return {?}
     */
    set adapter(adapter) {
        this.adapterStrategy = adapter;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.changeRef.detectChanges();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.listenerFn) {
            this.listenerFn();
        }
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.onTouch();
            this.updateInnerValue(event.target.value);
            this.valueChange.emit(this.innerDateValue);
            this._element.nativeElement.checkValidity();
            this.changeRef.markForCheck();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onCalendarSelect(event) {
        this.onTouch();
        this.updateInnerValue(event.detail);
        this.valueChange.emit(this.innerDateValue);
        this._element.nativeElement.checkValidity();
        this.changeRef.markForCheck();
    }
    // convert the masked value to storage value
    /**
     * @private
     * @param {?} inputDate
     * @return {?}
     */
    updateInnerValue(inputDate) {
        this.innerDateValue = this.adapterStrategy.convertToDate(inputDate);
        this.propogateInnerValueChange();
    }
    /**
     * @private
     * @return {?}
     */
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    }
}
DatePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatePicker]',
                exportAs: 'anjDatepickerInput'
            },] }
];
/** @nocollapse */
DatePickerDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: DateAdapter },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];
DatePickerDirective.propDecorators = {
    value: [{ type: Input }],
    adapter: [{ type: Input }],
    ngModel: [{ type: Input }],
    outerValue: [{ type: HostBinding, args: ['value',] }],
    languageCode: [{ type: HostBinding, args: ['attr.language',] }],
    onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
    onCalendarSelect: [{ type: HostListener, args: ['anjCalendarSelection', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class TextValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
        if (this.element.nativeElement.invalid) {
            this.checkValidity();
        }
    }
}
TextValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input, anj-textarea',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: TextValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
TextValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
TextValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.detail.value'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class RequiredCheckDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.requiredCheck(c);
    }
}
RequiredCheckDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class SelectValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     */
    constructor(parentForm, parentFormGroup, element) {
        super(element, parentForm, parentFormGroup);
    }
    /**
     * @param {?} target
     * @return {?}
     */
    onAnjChange(target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
        this.checkValidity();
    }
}
SelectValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-radio-group, anj-select, anj-type-ahead',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: SelectValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
SelectValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef }
];
SelectValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TimePickerDirective {
    /**
     * @param {?} _element
     * @param {?=} ngControl
     * @param {?=} _locale
     * @param {?=} _timezone
     * @param {?=} _meridiem
     */
    constructor(_element, ngControl, _locale, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._locale = _locale;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        const picker = this._element.nativeElement.querySelector('anj-time-picker');
        picker.setAttribute('locale', this._locale);
        picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            picker.setAttribute('timezone', this._timezone);
        }
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    }
}
TimePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjTimePicker]',
                exportAs: 'anjTimepickerInput'
            },] }
];
/** @nocollapse */
TimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
TimePickerDirective.propDecorators = {
    value: [{ type: HostBinding, args: ['value',] }],
    onInput: [{ type: HostListener, args: ['anjClockSelection', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ZuoraDatePipe {
    /**
     * @param {?=} _locale
     */
    constructor(_locale) {
        this._locale = _locale;
    }
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    transform(value, args) {
        try {
            /** @type {?} */
            const valueAsDate = new Date(value);
            /** @type {?} */
            const userOpts = (/** @type {?} */ (args));
            /** @type {?} */
            const defaultOpts = { year: '2-digit', month: 'numeric', day: 'numeric' };
            /** @type {?} */
            const opts = Object.assign({}, defaultOpts, userOpts);
            return new Intl.DateTimeFormat(this._locale, opts).format(valueAsDate);
        }
        catch (e) {
            return value.toString();
        }
    }
}
ZuoraDatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'zuoraDate'
            },] }
];
/** @nocollapse */
ZuoraDatePipe.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ToastService {
    constructor() {
        this._dismissed$ = new Subject();
    }
    /**
     * @return {?}
     */
    get dismissed$() {
        return this._dismissed$.asObservable();
    }
    /**
     * @return {?}
     */
    static createController() {
        /** @type {?} */
        const controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return (/** @type {?} */ (controller));
    }
    /**
     * @return {?}
     */
    dismissAll() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.dismissAll();
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    error(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.error(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    message(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.message(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    success(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.success(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    warn(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.warn(message, header);
        });
    }
    /**
     * @return {?}
     */
    getController() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.controller || this.findController();
        });
    }
    /**
     * @private
     * @return {?}
     */
    findController() {
        /** @type {?} */
        const controller = this.findOrCreate(document.querySelector('anj-toast-controller'));
        controller.addEventListener('anjToastDidDismiss', this.onToastDidDismiss.bind(this));
        this.controller = controller;
        return controller.componentOnReady();
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onToastDidDismiss(event) {
        this._dismissed$.next(event);
    }
    /**
     * @private
     * @param {?} control
     * @return {?}
     */
    findOrCreate(control) {
        return control || ToastService.createController();
    }
}
ToastService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ ToastService.ngInjectableDef = defineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LocalStorageService extends LocalStorage {
}
LocalStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ LocalStorageService.ngInjectableDef = defineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
class SessionStorageService extends SessionStorage {
}
SessionStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ SessionStorageService.ngInjectableDef = defineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class CustomNativeErrorsDirective {
    /**
     * @param {?} el
     * @param {?} changeRef
     */
    constructor(el, changeRef) {
        this.el = el;
        this.changeRef = changeRef;
    }
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.el.nativeElement.checkCustomValidity) {
                return null;
            }
            /** @type {?} */
            const validityErrors = yield this.el.nativeElement.checkCustomValidity();
            if (!validityErrors) {
                return null;
            }
            else {
                this.changeRef.markForCheck();
                return validityErrors;
            }
        });
    }
}
CustomNativeErrorsDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatePicker], anj-input[anjTimePicker]',
                providers: [
                    { provide: NG_ASYNC_VALIDATORS, useExisting: CustomNativeErrorsDirective, multi: true }
                ]
            },] }
];
/** @nocollapse */
CustomNativeErrorsDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatetimePickerDirective {
    /**
     * @param {?} _element
     * @param {?} changeRef
     * @param {?=} ngControl
     * @param {?=} _locale
     * @param {?=} _timezone
     * @param {?=} _meridiem
     */
    constructor(_element, changeRef, ngControl, _locale, _timezone, _meridiem) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this._locale = _locale;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.languageCode = (_locale) ? _locale.split('-')[0] : 'en';
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.picker = this._element.nativeElement.querySelector('anj-datetime-picker');
        this.picker.setAttribute('locale', this._locale);
        this.picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            this.picker.setAttribute('timezone', this._timezone);
        }
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.innerDateValue = this.ngModel || this.ngControl.value;
            /** @type {?} */
            const d = new Date(this.innerDateValue);
            this.outerValue = this.localizeForOuterValue(d);
            this.changeRef.detectChanges();
        }
    }
    // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.innerDateValue = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.picker.processInputValue(event.target.value);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onAnjDatetimeChange(event) {
        this.changeHandler(event.detail);
    }
    /**
     * @private
     * @param {?} changedVal
     * @return {?}
     */
    changeHandler(changedVal) {
        this.onTouch();
        this.outerValue = changedVal.outer;
        this.innerDateValue = changedVal.inner;
        this.propogateInnerValueChange();
        this.changeRef.markForCheck();
    }
    /**
     * @private
     * @param {?} date
     * @return {?}
     */
    localizeForOuterValue(date) {
        return date.toLocaleString(this._locale, { hour12: !!this._meridiem, month: '2-digit', day: '2-digit', year: 'numeric', hour: '2-digit', minute: '2-digit' });
    }
    /**
     * @private
     * @return {?}
     */
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
    }
}
DatetimePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatetimePicker]',
                exportAs: 'anjDatetimePickerInput'
            },] }
];
/** @nocollapse */
DatetimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
DatetimePickerDirective.propDecorators = {
    ngModel: [{ type: Input }],
    outerValue: [{ type: HostBinding, args: ['value',] }],
    languageCode: [{ type: HostBinding, args: ['attr.language',] }],
    onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
    onAnjDatetimeChange: [{ type: HostListener, args: ['anjDatetimeChange', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    CustomNativeErrorsDirective,
    DatePickerDirective,
    DatetimePickerDirective,
    DateValidatorDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
class AnjunaCoreModule {
}
AnjunaCoreModule.decorators = [
    { type: NgModule, args: [{
                declarations: DECLARATIONS,
                exports: DECLARATIONS,
                providers: [
                    {
                        provide: DateAdapter,
                        useClass: NativeDateAdapter,
                        deps: [LOCALE_ID]
                    },
                    {
                        provide: TIME_MERIDIEM,
                        useValue: true
                    },
                    ZuoraDatePipe
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BaseTranslationPipe, DraggableDirective, DroppableDirective, AnjTable, TemplateDirective, TransitionDirective, BooleanValueDirective, DateValidatorDirective, DatePickerDirective, TextValueDirective, RequiredCheckDirective, SelectValueDirective, TimePickerDirective, LANGUAGE_CODE, TIME_TIMEZONE_STRING, TIME_MERIDIEM, DATE_TIMEZONE_OVERRIDE, DomOutlet, Template, AnjunaValidators, ZuoraDatePipe, ToastService, LocalStorageService, SessionStorageService, getWeekdays, getMonths, localize, parseInput, STANDARD_LOCALES, DateAdapter, NativeDateAdapter, TransitionState, AnjunaCoreModule, CustomNativeErrorsDirective as ɵb, DatetimePickerDirective as ɵc, ValueAccessor as ɵa };

//# sourceMappingURL=anjuna-angular-core.js.map